@@ -1,9 +1,8 @@
 #include "tintin.h"
 #include "protos/glob.h"
-#include "protos/llist.h"
 #include "protos/utils.h"
 
-#define DELETED_HASHENTRY ((char*)init_hash)
+#define DELETED_HASHENTRY ((char*)-1)
 
 // Jenkins hash
 static uint32_t hash(const char *key)
@@ -105,13 +104,11 @@ void set_hash(struct hashtable *h, const char *key, const char *value)
 {
     if (h->nent*5 > h->size*4)
         rehash(h, h->nval*3);
-    int j=-1;
     int i=hash(key)%h->size;
     while (h->tab[i].left)
     {
         if (h->tab[i].left==DELETED_HASHENTRY)
-            if (j==-1)
-                j=i;
+            goto found_tombstone;
         if (!strcmp(h->tab[i].left, key))
         {
             SFREE(h->tab[i].right);
@@ -122,10 +119,8 @@ void set_hash(struct hashtable *h, const char *key, const char *value)
             i=h->size;
         i--;
     }
-    if (j!=-1)
-        i=j;
-    else
-        h->nent++;
+    h->nent++;
+found_tombstone:
     h->tab[i].left = mystrdup(key);
     h->tab[i].right= mystrdup(value);
     h->nval++;
@@ -136,13 +131,11 @@ void set_hash_nostring(struct hashtable *h, const char *key, const char *value)
 {
     if (h->nent*5 > h->size*4)
         rehash(h, h->nval*3);
-    int j=-1;
     int i=hash(key)%h->size;
     while (h->tab[i].left)
     {
         if (h->tab[i].left==DELETED_HASHENTRY)
-            if (j==-1)
-                j=i;
+            goto found_tombstone;
         if (!strcmp(h->tab[i].left, key))
         {
             h->tab[i].right=(char*)value;
@@ -152,10 +145,8 @@ void set_hash_nostring(struct hashtable *h, const char *key, const char *value)
             i=h->size;
         i--;
     }
-    if (j!=-1)
-        i=j;
-    else
-        h->nent++;
+    h->nent++;
+found_tombstone:
     h->tab[i].left = mystrdup(key);
     h->tab[i].right= (char*)value;
     h->nval++;
@@ -208,47 +199,6 @@ bool delete_hash(struct hashtable *h, const char *key)
 }
 
 
-/*****************************************************/
-/* merge two sorted llists (without heads!) into one */
-/*****************************************************/
-static struct listnode* merge_lists(struct listnode* a, struct listnode* b)
-{
-    struct listnode* c=0, *c0;
-
-    if (!a)
-        return b;
-    if (!b)
-        return a;
-    if (strcmp(a->left, b->left)<=0)
-    {
-        c0=c=a;
-        a=a->next;
-    }
-    else
-    {
-        c0=c=b;
-        b=b->next;
-    }
-    while (a && b)
-        if (strcmp(a->left, b->left)<=0)
-        {
-            c->next=a;
-            c=a;
-            a=a->next;
-        }
-        else
-        {
-            c->next=b;
-            c=b;
-            b=b->next;
-        }
-    if (a)
-        c->next=a;
-    else
-        c->next=b;
-    return c0;
-}
-
 /**************************************************************************/
 /* create a sorted llist containing all entries of the table matching pat */
 /**************************************************************************/
@@ -256,38 +206,33 @@ static struct listnode* merge_lists(struct listnode* a, struct listnode* b)
 /* deleting from a list, we should show the entries in a sorted order,    */
 /* however screen output is slow anyways, so we can sort it on the fly.   */
 /**************************************************************************/
-struct listnode* hash2list(struct hashtable *h, const char *pat)
+static int paircmp(const void *a, const void *b)
 {
-#define NBITS ((int)sizeof(void*)*8)
-    struct listnode *p[NBITS];     /* polynomial sort, O(n*log(n)) */
-    struct listnode *l;
+    return strcmp(((struct pair*)a)->left, ((struct pair*)b)->left);
+}
+
+typedef int (*compar_t)(const void*, const void*);
+
+struct pairlist* hash2list(struct hashtable *h, const char *pat)
+{
+    int n = h->nval;
+    struct pairlist *pl = MALLOC((n*2+1)*sizeof(void*));
+    struct pair *p = &pl->pairs[0];
 
-    for (int j=0;j<NBITS;j++)
-        p[j]=0;
     for (int i=0;i<h->size;i++)
         if (h->tab[i].left && (h->tab[i].left!=DELETED_HASHENTRY)
-            && match(pat, h->tab[i].left))
+            && (!pat || match(pat, h->tab[i].left)))
         {
-            if (!(l=TALLOC(struct listnode)))
-                syserr("couldn't malloc listhead");
-            l->left = h->tab[i].left;
-            l->right= h->tab[i].right;
-            l->pr   = 0;
-            l->next = 0;
-            int j;
-            for (j=0; p[j]; j++)     /* if j>=NBITS, we have a bug anyway */
-            {
-                l=merge_lists(p[j], l);
-                p[j]=0;
-            }
-            p[j]=l;
+            p->left = h->tab[i].left;
+            p->right= h->tab[i].right;
+            p++;
         }
-    l=0;
-    for (int j=0; j<NBITS; j++)
-        l=merge_lists(p[j], l);
-    p[0]=init_list();
-    p[0]->next=l;
-    return p[0];
+
+    n = p-&pl->pairs[0];
+    qsort(&pl->pairs[0], n, sizeof(struct pair), paircmp);
+    pl->size = n;
+
+    return pl;
 }
 
 
