@@ -43,11 +43,10 @@ static bool check_regexp(char *line, char *action, pvars_t *vars, struct session
 /*********************/
 /* the #grep command */
 /*********************/
-void grep_command(const char *arg, struct session *ses)
+struct session *grep_command(const char *arg, struct session *ses)
 {
     pvars_t vars, *lastpvars;
     char left[BUFFER_SIZE], line[BUFFER_SIZE], right[BUFFER_SIZE];
-    bool flag=false;
 
     arg=get_arg(arg, left, 0, ses);
     arg=get_arg(arg, line, 0, ses);
@@ -56,36 +55,19 @@ void grep_command(const char *arg, struct session *ses)
     if (!*left || !*right)
     {
         tintin_eprintf(ses, "#ERROR: valid syntax is: #grep <pattern> <line> <command> [#else ...]");
-        return;
+        return ses;
     }
 
     if (check_regexp(line, left, &vars, ses))
     {
         lastpvars = pvars;
         pvars = &vars;
-        parse_input(right, true, ses);
+        ses = parse_input(right, true, ses);
         pvars = lastpvars;
-        flag=true;
+        return ses;
     }
-    arg=get_arg_in_braces(arg, left, 0);
-    if (*left == tintin_char)
-    {
-        if (is_abrev(left + 1, "else"))
-        {
-            get_arg_in_braces(arg, right, 1);
-            if (!flag)
-                parse_input(right, true, ses);
-            return;
-        }
-        if (is_abrev(left + 1, "elif"))
-        {
-            if (!flag)
-                if_command(arg, ses);
-            return;
-        }
-    }
-    if (*left)
-        tintin_eprintf(ses, "#ERROR: cruft after #grep: {%s}", left);
+
+    return ifelse("grep", arg, ses);
 }
 
 
