@@ -26,6 +26,8 @@ static void list_subs(const char *left, bool gag, struct session *ses)
     kbtree_t(trip) *sub = ses->subs;
 
     TRIP_ITER(sub, mysubs)
+        if (!match(left, mysubs->left))
+            continue;
         if (gag)
         {
             if (!strcmp(mysubs->right, EMPTY_LINE))
@@ -41,17 +43,17 @@ static void list_subs(const char *left, bool gag, struct session *ses)
             if (!flag)
                 tintin_printf(ses, "#THESE SUBSTITUTES HAVE BEEN DEFINED:");
             flag=true;
-            show_trip(mysubs);
+            show_trip(mysubs, ses);
         }
     ENDITER
 
-    if (!flag && ses->mesvar[MSG_SUBSTITUTE])
-    {
-        if (strcmp(left, "*"))
-            tintin_printf(ses, "#THAT %s IS NOT DEFINED.", gag? "GAG":"SUBSTITUTE");
-        else
-            tintin_printf(ses, "#NO %sS HAVE BEEN DEFINED.", gag? "GAG":"SUBSTITUTE");
-    }
+    if (flag)
+        return;
+
+    if (strcmp(left, "*"))
+        tintin_printf(ses, "#THAT %s IS NOT DEFINED.", gag? "GAG":"SUBSTITUTE");
+    else
+        tintin_printf(ses, "#NO %sS HAVE BEEN DEFINED.", gag? "GAG":"SUBSTITUTE");
 }
 
 static void parse_sub(const char *left_, const char *right,  bool gag, struct session *ses)
@@ -79,7 +81,7 @@ static void parse_sub(const char *left_, const char *right,  bool gag, struct se
     }
     kb_put(trip, sub, new);
     subnum++;
-#ifdef HAVE_HS
+#ifdef HAVE_SIMD
     ses->subs_dirty=true;
 #endif
     if (ses->mesvar[MSG_SUBSTITUTE])
@@ -131,13 +133,13 @@ static void unsub(const char *arg, bool gag, struct session *ses)
     if (!delete_tlist(ses->subs, left, ses->mesvar[MSG_SUBSTITUTE]?
         gag? "#Ok. {%s} is no longer gagged." :
         "#Ok. {%s} is no longer substituted." : 0,
-        gag? is_not_gag : is_gag, true)
+        gag? is_not_gag : is_gag, true, ses)
         && ses->mesvar[MSG_SUBSTITUTE])
     {
         tintin_printf(ses, "#THAT SUBSTITUTE (%s) IS NOT DEFINED.", left);
     }
 
-#ifdef HAVE_HS
+#ifdef HAVE_SIMD
     ses->subs_dirty=true;
 #endif
 }
@@ -212,7 +214,7 @@ static void do_all_sub_serially(char *line, struct session *ses)
     pvars=lastpvars;
 }
 
-#ifdef HAVE_HS
+#ifdef HAVE_SIMD
 // Hyperscan disallows patterns that match an empty buffer, we do them by hand.
 static bool is_omni_regex(const char *pat)
 {
@@ -244,7 +246,7 @@ static void build_subs_hs(struct session *ses)
     uintptr_t *markers = MALLOC(n*sizeof(uintptr_t));
 
     if (!pat || !flags || !ids || !data || !markers)
-        syserr("out of memory");
+        die("out of memory");
 
     ses->subs_omni_last=ses->subs_omni_first=n;
 
@@ -291,7 +293,7 @@ done:
     MFREE(pat, n*sizeof(void*));
 
     if (ses->subs_hs && hs_alloc_scratch(ses->subs_hs, &hs_scratch))
-        syserr("out of memory");
+        die("out of memory");
 }
 
 static unsigned short longest_len[BUFFER_SIZE];
@@ -358,7 +360,7 @@ gagged:
 
 void do_all_sub(char *line, struct session *ses)
 {
-#ifdef HAVE_HS
+#ifdef HAVE_SIMD
     if (!kb_size(ses->subs))
         return;
 
